@@ -15,7 +15,7 @@ ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
-DOCUMENTATION = """
+DOCUMENTATION = r'''
 ---
 module: cron
 short_description: Manage cron.d and crontab entries
@@ -27,66 +27,78 @@ description:
     which is used by future ansible/module calls to find/check the state. The "name"
     parameter should be unique, and changing the "name" value will result in a new cron
     task being created (or a different one being removed).'
-  - 'When environment variables are managed: no comment line is added, but, when the module
+  - When environment variables are managed, no comment line is added, but, when the module
     needs to find/check the state, it uses the "name" parameter to find the environment
-    variable definition line.'
-  - 'When using symbols such as %, they must be properly escaped.'
+    variable definition line.
+  - When using symbols such as %, they must be properly escaped.
 version_added: "0.9"
 options:
   name:
     description:
       - Description of a crontab entry or, if env is set, the name of environment variable.
-        Required if state=absent. Note that if name is not set and state=present, then a
+      - Required if C(state=absent).
+      - Note that if name is not set and C(state=present), then a
         new crontab entry will always be created, regardless of existing ones.
+      - This parameter will always be required in future releases.
+    type: str
   user:
     description:
       - The specific user whose crontab should be modified.
-    default: root
+      - When unset, this parameter defaults to using C(root).
+    type: str
   job:
     description:
       - The command to execute or, if env is set, the value of environment variable.
-        The command should not contain line breaks.
-        Required if state=present.
+      - The command should not contain line breaks.
+      - Required if C(state=present).
+    type: str
     aliases: [ value ]
   state:
     description:
       - Whether to ensure the job or environment variable is present or absent.
+    type: str
     choices: [ absent, present ]
     default: present
   cron_file:
     description:
       - If specified, uses this file instead of an individual user's crontab.
-        If this is a relative path, it is interpreted with respect to
-        /etc/cron.d. (If it is absolute, it will typically be /etc/crontab).
-        Many linux distros expect (and some require) the filename portion to consist solely
+      - If this is a relative path, it is interpreted with respect to I(/etc/cron.d).
+      - If it is absolute, it will typically be I(/etc/crontab).
+      - Many linux distros expect (and some require) the filename portion to consist solely
         of upper- and lower-case letters, digits, underscores, and hyphens.
-        To use the C(cron_file) parameter you must specify the C(user) as well.
+      - To use the C(cron_file) parameter you must specify the C(user) as well.
+    type: str
   backup:
     description:
       - If set, create a backup of the crontab before it is modified.
         The location of the backup is returned in the C(backup_file) variable by this module.
     type: bool
-    default: 'no'
+    default: no
   minute:
     description:
       - Minute when the job should run ( 0-59, *, */2, etc )
+    type: str
     default: "*"
   hour:
     description:
       - Hour when the job should run ( 0-23, *, */2, etc )
+    type: str
     default: "*"
   day:
     description:
       - Day of the month the job should run ( 1-31, *, */2, etc )
+    type: str
     default: "*"
     aliases: [ dom ]
   month:
     description:
       - Month of the year the job should run ( 1-12, *, */2, etc )
+    type: str
     default: "*"
   weekday:
     description:
       - Day of the week that the job should run ( 0-6 for Sunday-Saturday, *, etc )
+    type: str
     default: "*"
     aliases: [ dow ]
   reboot:
@@ -94,99 +106,100 @@ options:
       - If the job should be run at reboot. This option is deprecated. Users should use special_time.
     version_added: "1.0"
     type: bool
-    default: "no"
+    default: no
   special_time:
     description:
       - Special time specification nickname.
-    choices: [ reboot, yearly, annually, monthly, weekly, daily, hourly ]
+    type: str
+    choices: [ annually, daily, hourly, monthly, reboot, weekly, yearly ]
     version_added: "1.3"
   disabled:
     description:
       - If the job should be disabled (commented out) in the crontab.
       - Only has effect if C(state=present).
     type: bool
-    default: 'no'
+    default: no
     version_added: "2.0"
   env:
     description:
-      - If set, manages a crontab's environment variable. New variables are added on top of crontab.
-        "name" and "value" parameters are the name and the value of environment variable.
+      - If set, manages a crontab's environment variable.
+      - New variables are added on top of crontab.
+      - C(name) and C(value) parameters are the name and the value of environment variable.
     type: bool
-    default: "no"
+    default: no
     version_added: "2.1"
   insertafter:
     description:
-      - Used with C(state=present) and C(env). If specified, the environment variable will be
-        inserted after the declaration of specified environment variable.
+      - Used with C(state=present) and C(env).
+      - If specified, the environment variable will be inserted after the declaration of specified environment variable.
+    type: str
     version_added: "2.1"
   insertbefore:
     description:
-      - Used with C(state=present) and C(env). If specified, the environment variable will be
-        inserted before the declaration of specified environment variable.
+      - Used with C(state=present) and C(env).
+      - If specified, the environment variable will be inserted before the declaration of specified environment variable.
+    type: str
     version_added: "2.1"
 requirements:
   - cron
 author:
     - Dane Summers (@dsummersl)
-    - Mike Grozak
-    - Patrick Callahan
+    - Mike Grozak (@rhaido)
+    - Patrick Callahan (@dirtyharrycallahan)
     - Evan Kaufman (@EvanK)
     - Luca Berruti (@lberruti)
-"""
+'''
 
-EXAMPLES = '''
-# Ensure a job that runs at 2 and 5 exists.
-# Creates an entry like "0 5,2 * * ls -alh > /dev/null"
-- cron:
+EXAMPLES = r'''
+- name: Ensure a job that runs at 2 and 5 exists. Creates an entry like "0 5,2 * * ls -alh > /dev/null"
+  cron:
     name: "check dirs"
     minute: "0"
     hour: "5,2"
     job: "ls -alh > /dev/null"
 
-# Ensure an old job is no longer present. Removes any job that is prefixed
-# by "#Ansible: an old job" from the crontab
-- cron:
+- name: 'Ensure an old job is no longer present. Removes any job that is prefixed by "#Ansible: an old job" from the crontab'
+  cron:
     name: "an old job"
     state: absent
 
-# Creates an entry like "@reboot /some/job.sh"
-- cron:
+- name: Creates an entry like "@reboot /some/job.sh"
+  cron:
     name: "a job for reboot"
     special_time: reboot
     job: "/some/job.sh"
 
-# Creates an entry like "PATH=/opt/bin" on top of crontab
-- cron:
+- name: Creates an entry like "PATH=/opt/bin" on top of crontab
+  cron:
     name: PATH
     env: yes
-    value: /opt/bin
+    job: /opt/bin
 
-# Creates an entry like "APP_HOME=/srv/app" and insert it after PATH
-# declaration
-- cron:
+- name: Creates an entry like "APP_HOME=/srv/app" and insert it after PATH declaration
+  cron:
     name: APP_HOME
     env: yes
-    value: /srv/app
+    job: /srv/app
     insertafter: PATH
 
-# Creates a cron file under /etc/cron.d
-- cron:
+- name: Creates a cron file under /etc/cron.d
+  cron:
     name: yum autoupdate
     weekday: 2
     minute: 0
     hour: 12
     user: root
-    job: "YUMINTERACTIVE: 0 /usr/sbin/yum-autoupdate"
+    job: "YUMINTERACTIVE=0 /usr/sbin/yum-autoupdate"
     cron_file: ansible_yum-autoupdate
 
-# Removes a cron file from under /etc/cron.d
-- cron:
+- name: Removes a cron file from under /etc/cron.d
+  cron:
     name: "yum autoupdate"
     cron_file: ansible_yum-autoupdate
     state: absent
 
-# Removes "APP_HOME" environment variable from crontab
-- cron:
+- name: Removes "APP_HOME" environment variable from crontab
+  cron:
     name: APP_HOME
     env: yes
     state: absent
@@ -249,7 +262,7 @@ class CronTab(object):
             except IOError:
                 # cron file does not exist
                 return
-            except:
+            except Exception:
                 raise CronTabError("Unexpected error:", sys.exc_info()[0])
         else:
             # using safely quoted shell for now, but this really should be two non-shell calls instead.  FIXME
@@ -374,7 +387,7 @@ class CronTab(object):
         except OSError:
             # cron file does not exist
             return False
-        except:
+        except Exception:
             raise CronTabError("Unexpected error:", sys.exc_info()[0])
 
     def find_job(self, name, job=None):
@@ -569,6 +582,12 @@ def main():
             ['reboot', 'special_time'],
             ['insertafter', 'insertbefore'],
         ],
+        required_by=dict(
+            cron_file=('user',),
+        ),
+        required_if=(
+            ('state', 'present', ('job',)),
+        ),
     )
 
     name = module.params['name']
@@ -606,6 +625,17 @@ def main():
 
     module.debug('cron instantiated - name: "%s"' % name)
 
+    if not name:
+        module.deprecate(
+            msg="The 'name' parameter will be required in future releases.",
+            version='2.12'
+        )
+    if reboot:
+        module.deprecate(
+            msg="The 'reboot' parameter will be removed in future releases. Use 'special_time' option instead.",
+            version='2.12'
+        )
+
     if module._diff:
         diff = dict()
         diff['before'] = crontab.existing
@@ -627,13 +657,6 @@ def main():
     if (special_time or reboot) and get_platform() == 'SunOS':
         module.fail_json(msg="Solaris does not support special_time=... or @reboot")
 
-    if cron_file and do_install:
-        if not user:
-            module.fail_json(msg="To use cron_file=... parameter you must specify user=... as well")
-
-    if job is None and do_install:
-        module.fail_json(msg="You must specify 'job' to install a new cron job or variable")
-
     if (insertafter or insertbefore) and not env and do_install:
         module.fail_json(msg="Insertafter and insertbefore parameters are valid only with env=yes")
 
@@ -701,7 +724,7 @@ def main():
                 changed = True
 
     # no changes to env/job, but existing crontab needs a terminating newline
-    if not changed and not crontab.existing == '':
+    if not changed and crontab.existing != '':
         if not (crontab.existing.endswith('\r') or crontab.existing.endswith('\n')):
             changed = True
 
